USE AdventureWorks2014;
SET NOCOUNT ON;
GO

-- typical first try with FOR XML PATH:
SELECT BusinessEntityID, LastName, 
  FirstNames = 
  (
    SELECT ',' + FirstName
    FROM Person.Person 
	WHERE LastName = p.LastName
    FOR XML PATH('')
  )
FROM Person.Person AS p
ORDER BY LastName, FirstNames;

-- Above has duplicate first names in comma-separated list
-- Let's remove those in the subquery
SELECT BusinessEntityID, LastName, 
  FirstNames = 
  (
    SELECT DISTINCT ',' + FirstName
    FROM Person.Person 
	WHERE LastName = p.LastName
    FOR XML PATH('')
  )
FROM Person.Person AS p
ORDER BY LastName, FirstNames;

-- But we also have duplicate last names
-- represented by different BusinessEntityIDs
-- (People with same last name, and in some
-- cases, the same first name, too). Let's 
-- drop the BusinessEntityID column and slap
-- a DISTINCT on the outer query too:

SELECT DISTINCT LastName, 
  FirstNames = 
  (
    SELECT DISTINCT ',' + FirstName
    FROM Person.Person 
	WHERE LastName = p.LastName
    FOR XML PATH('')
  )
FROM Person.Person AS p
ORDER BY LastName, FirstNames;

-- above will still fail if there are XML-unfriendly
-- entities in the data (these won't always be names)
UPDATE Person.Person SET FirstName += '>' WHERE BusinessEntityID = 285;

-- forcing XML PATH, TYPE and taking the first (only)
-- value prevents the XML characters from entitization:
SELECT DISTINCT LastName, 
  FirstNames = 
  (
    SELECT DISTINCT ',' + FirstName
    FROM Person.Person 
	WHERE LastName = p.LastName
    FOR XML PATH, TYPE
  ).value('.[1]','nvarchar(max)')
FROM Person.Person AS p
ORDER BY LastName, FirstNames;

-- let's fix that name:
UPDATE Person.Person SET FirstName = REPLACE(FirstName, '>', '')
  WHERE BusinessEntityID = 285;

-- Now, the order of the FirstName in the 
-- comma-separated list is not guaranteed.
-- Let's make sure, by adding ORDER BY in
-- the subquery. Also, no longer need outer
-- ORDER BY on FirstNames, since each last
-- name is only represented once.
SELECT DISTINCT LastName, 
  FirstNames = 
  (
    SELECT DISTINCT ',' + FirstName
    FROM Person.Person 
	WHERE LastName = p.LastName
	ORDER BY ',' + FirstName
    FOR XML PATH, TYPE
  ).value('.[1]','nvarchar(max)')
FROM Person.Person AS p
ORDER BY LastName;

-- Finally, that leading comma is ugly. We
-- can get rid of it using RIGHT() but then
-- we need to grab the length of the string
-- and subtract 1. Instead, lets's use STUFF:
SELECT DISTINCT LastName, 
  FirstNames = 
  STUFF((
    SELECT DISTINCT ',' + FirstName
    FROM Person.Person 
	WHERE LastName = p.LastName
	ORDER BY ',' + FirstName
    FOR XML PATH, TYPE
  ).value('.[1]','nvarchar(max)'),1,1,'')
FROM Person.Person AS p
ORDER BY LastName;

